package com.zhongzheng.modules.exam.mapper;

import com.zhongzheng.modules.exam.bo.ExamBeforeQueryBo;
import com.zhongzheng.modules.exam.bo.ExamNumberGoodsQueryBo;
import com.zhongzheng.modules.exam.domain.ExamBefore;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.zhongzheng.modules.exam.vo.ExamBeforeVo;
import com.zhongzheng.modules.exam.vo.ExamNumberGoodsVo;

import java.util.List;

/**
 * 前培安排Mapper接口
 *
 * @author ruoyi
 * @date 2021-12-07
 */
public interface ExamBeforeMapper extends BaseMapper<ExamBefore> {

    List<ExamNumberGoodsVo> listGoods(ExamNumberGoodsQueryBo bo);

    List<ExamNumberGoodsVo> examNumberGoodsVos(Long beforeId);

    ExamBeforeVo selectByBefore(Long beforeId);

    List<ExamBeforeVo> queryBefore(ExamBeforeQueryBo bo);

    Integer countBeforeGoods(Long beforeId);

    Integer beforeGoods(Long beforeId);

    Long verifyInfo(Long goodsId);

    ExamBeforeVo verifyExamBeforeVo(ExamBeforeQueryBo examBeforeQueryBo);
}
