package com.zhongzheng.modules.exam.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.zhongzheng.common.utils.DateUtils;
import com.zhongzheng.common.utils.ServletUtils;
import com.zhongzheng.modules.collect.domain.CollectBank;
import com.zhongzheng.modules.exam.bo.*;
import com.zhongzheng.modules.exam.vo.ExamNumberGoodsVo;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.pagehelper.Page;
import com.zhongzheng.modules.exam.domain.ExamBefore;
import com.zhongzheng.modules.exam.mapper.ExamBeforeMapper;
import com.zhongzheng.modules.exam.vo.ExamBeforeVo;
import com.zhongzheng.modules.exam.service.IExamBeforeService;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 前培安排Service业务层处理
 *
 * @author ruoyi
 * @date 2021-12-07
 */
@Service
public class ExamBeforeServiceImpl extends ServiceImpl<ExamBeforeMapper, ExamBefore> implements IExamBeforeService {

    @Override
    public ExamBeforeVo queryById(Long beforeId){
        ExamBeforeVo examBeforeVo = baseMapper.selectByBefore(beforeId);
        List<ExamNumberGoodsVo> examNumberGoodsVos = baseMapper.examNumberGoodsVos(beforeId);
        examBeforeVo.setExamNumberGoods(examNumberGoodsVos);
        return examBeforeVo;
    }

    @Override
    public List<ExamBeforeVo> queryList(ExamBeforeQueryBo bo) {
        List<ExamBeforeVo> examBeforeVos = baseMapper.queryBefore(bo);
        for (ExamBeforeVo examBeforeVo : examBeforeVos) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTimeZone(TimeZone.getTimeZone("GMT+8:00"));
            calendar.setTimeInMillis(System.currentTimeMillis());
            calendar.set(Calendar.HOUR_OF_DAY, 23);
            calendar.set(Calendar.MINUTE, 59);
            calendar.set(Calendar.SECOND, 59);
            calendar.set(Calendar.MILLISECOND, 999);
            if ((examBeforeVo.getBeforeEndTime()+86400)  < calendar.getTimeInMillis()/1000){
                ExamBefore update = new ExamBefore();
                update.setBeforeId(examBeforeVo.getBeforeId());
                update.setStatus(2);
                update.setUpdateTime(DateUtils.getNowTime());
                this.updateById(update);
            }
        }
        return baseMapper.queryBefore(bo);
    }

    /**
    * 实体类转化成视图对象
    *
    * @param collection 实体类集合
    * @return
    */
    private List<ExamBeforeVo> entity2Vo(Collection<ExamBefore> collection) {
        List<ExamBeforeVo> voList = collection.stream()
                .map(any -> BeanUtil.toBean(any, ExamBeforeVo.class))
                .collect(Collectors.toList());
        if (collection instanceof Page) {
            Page<ExamBefore> page = (Page<ExamBefore>)collection;
            Page<ExamBeforeVo> pageVo = new Page<>();
            BeanUtil.copyProperties(page,pageVo);
            pageVo.addAll(voList);
            voList = pageVo;
        }
        return voList;
    }

    @Override
    public Boolean insertByAddBo(ExamBeforeAddBo bo) {
        ExamBefore add = BeanUtil.toBean(bo, ExamBefore.class);
        List<ExamBefore> list = this.list(new LambdaQueryWrapper<ExamBefore>()
                .eq(ExamBefore::getBeforeName, add.getBeforeName())
                .in(ExamBefore::getStatus, 1,0));
        if (CollectionUtils.isNotEmpty(list)){
            throw new IllegalArgumentException("前培标题不可重复");
        }
        validEntityBeforeSave(add);
        add.setCode(ServletUtils.getEncoded("QP"));
        add.setCreateTime(DateUtils.getNowTime());
        add.setUpdateTime(DateUtils.getNowTime());
        return this.save(add);
    }

    @Override
    public Boolean updateByEditBo(ExamBeforeEditBo bo) {
        ExamBefore update = BeanUtil.toBean(bo, ExamBefore.class);
        if (bo.getStatus() == 1) {
            if (baseMapper.countBeforeGoods(bo.getBeforeId()) < 1) {
                throw new IllegalArgumentException("请先设置适用商品");
            }
            if (baseMapper.beforeGoods(bo.getBeforeId()) > 0){
                throw new IllegalArgumentException("该商品已启用考培，请重新创建");
            }
        }
        if (bo.getStatus() != -1){
            List<ExamBefore> list = this.list(new LambdaQueryWrapper<ExamBefore>()
                    .eq(ExamBefore::getBeforeName, bo.getBeforeName())
                    .eq(ExamBefore::getBeforeId, update.getBeforeId()));
            if (CollectionUtils.isEmpty(list)){
                List<ExamBefore> examBefore = this.list(new LambdaQueryWrapper<ExamBefore>()
                        .eq(ExamBefore::getBeforeName, update.getBeforeName())
                        .in(ExamBefore::getStatus,new ArrayList<Integer>(Arrays.asList(1,0)) ));
                if (CollectionUtils.isNotEmpty(examBefore)){
                    throw new IllegalArgumentException("前培标题不可重复");
                }
            }
        }
        validEntityBeforeSave(update);
        update.setUpdateTime(DateUtils.getNowTime());
        return this.updateById(update);
    }

    /**
     * 保存前的数据校验
     *
     * @param entity 实体类数据
     */
    private void validEntityBeforeSave(ExamBefore entity){
        //TODO 做一些数据校验,如唯一约束
    }

    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return this.removeByIds(ids);
    }

    @Override
    public List<ExamNumberGoodsVo> listGoods(ExamNumberGoodsQueryBo bo) {
        return baseMapper.listGoods(bo);
    }

    @Override
    public ExamBeforeVo getGoodsInfo(ExamBeforeQueryBo bo) {
        ExamBeforeVo examBeforeVo = baseMapper.selectByBefore(bo.getBeforeId());
        List<ExamNumberGoodsVo> examNumberGoodsVos = baseMapper.examNumberGoodsVos(bo.getBeforeId());
        examBeforeVo.setExamNumberGoods(examNumberGoodsVos);
        //初始化需要得到的数组
        Long[] array = new Long[examNumberGoodsVos.size()];
        //使用for循环得到数组
        for(int i = 0; i < examNumberGoodsVos.size();i++){
            array[i] = examNumberGoodsVos.get(i).getGoodsId();
        }
        examBeforeVo.setGoodsId(array);
        return examBeforeVo;
    }

    @Override
    public ExamBeforeVo getAddressInfo(ExamBeforeQueryBo examBeforeQueryBo) {
        Long verLong = baseMapper.verifyInfo(examBeforeQueryBo.getGoodsId());
        if (verLong < 1){
            throw new IllegalArgumentException("您不符合考前刷题条件，请先预约考试，或者咨询相关工作人员，了解使用条件。");
        }
        ExamBeforeVo examBeforeVo = baseMapper.verifyExamBeforeVo(examBeforeQueryBo);
        return examBeforeVo;
    }
}
